/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
/** @file
 * @brief unit test for cron functions.
 *
 * @copyright Copyright &copy; 2009 by Intra2net AG
 *
 */

#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>

#include <time.h>
#include <stdlib.h>
#include <stdexcept>

#include <cron.hpp>

using namespace std;
using namespace I2n::Time;

class TestCronIntervalFuncFixture
{
public:
    TestCronIntervalFuncFixture()
    {
        // our check-values are set for Germany
        setenv("TZ",":Europe/Berlin",1);
    }
};

BOOST_FIXTURE_TEST_SUITE(TestCronIntervalFunc, TestCronIntervalFuncFixture)

BOOST_AUTO_TEST_CASE(IntervalBeginToday)
{
    WeekCron cron("2345",3600,7200,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233100800), cron.get_next_run(1233099657));
}

BOOST_AUTO_TEST_CASE(IntervalDoneToday)
{
    WeekCron cron("2345",3600);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233150000));
}

BOOST_AUTO_TEST_CASE(IntervalBeginNow)
{
    WeekCron cron("2345",3600,7200,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233100810), cron.get_next_run(1233100800));
}

BOOST_AUTO_TEST_CASE(IntervalBeginStep)
{
    WeekCron cron("2345",3600,7200,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233100820), cron.get_next_run(1233100810));
}

BOOST_AUTO_TEST_CASE(IntervalWithin)
{
    WeekCron cron("2345",3600,7200,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233100830), cron.get_next_run(1233100822));
}

BOOST_AUTO_TEST_CASE(IntervalLaststep1)
{
    WeekCron cron("2345",3600,7200,11);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233104399));
}

BOOST_AUTO_TEST_CASE(IntervalLaststep2)
{
    WeekCron cron("2345",3600,7200,11);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233104398));
}

BOOST_AUTO_TEST_CASE(IntervalLaststep3)
{
    WeekCron cron("2345",3600,7199,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233104397));
}

BOOST_AUTO_TEST_CASE(IntervalLaststep4)
{
    WeekCron cron("2345",3600,7200,3599);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233104399), cron.get_next_run(1233104398));
}

BOOST_AUTO_TEST_CASE(IntervalLaststep5)
{
    WeekCron cron("2345",3600,7200,3598);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233104398));
}

BOOST_AUTO_TEST_CASE(IntervalLaststepMatch)
{
    WeekCron cron("2345",3600,7200,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233104399));
}

BOOST_AUTO_TEST_CASE(IntervalEnd)
{
    WeekCron cron("2345",3600,7200,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233104400));
}

BOOST_AUTO_TEST_CASE(IntervalBigstep)
{
    WeekCron cron("2345",3600,7200,10000);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233100800));
}

BOOST_AUTO_TEST_CASE(IntervalWholeDayStart)
{
    WeekCron cron("345",0,86400,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233097200), cron.get_next_run(1233097199));
}

BOOST_AUTO_TEST_CASE(IntervalWholeDayStartNow)
{
    WeekCron cron("345",0,86400,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233097210), cron.get_next_run(1233097200));
}

BOOST_AUTO_TEST_CASE(IntervalWholeDayEnd)
{
    WeekCron cron("2",0,86400,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233615600), cron.get_next_run(1233097199));
}

BOOST_AUTO_TEST_CASE(IntervalWholeDayWithin1)
{
    WeekCron cron("2345",0,86400,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233097230), cron.get_next_run(1233097220));
}

BOOST_AUTO_TEST_CASE(IntervalWholeDayWithin2)
{
    WeekCron cron("2345",0,86400,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233097230), cron.get_next_run(1233097229));
}

BOOST_AUTO_TEST_CASE(IntervalWholeDayWithinBoundary)
{
    WeekCron cron("2345",0,86400,10);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233097200), cron.get_next_run(1233097199));
}

BOOST_AUTO_TEST_CASE(IntervalBeforeOnce)
{
    WeekCron cron("2345",0,3600,3600);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233183600), cron.get_next_run(1233100799));
}

BOOST_AUTO_TEST_CASE(IntervalBeforeOnceBig)
{
    WeekCron cron("2345",0,3600,86400);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233183600), cron.get_next_run(1233100799));
}

BOOST_AUTO_TEST_CASE(IntervalAfterOnce)
{
    WeekCron cron("2345",0,3600,86400);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233183600), cron.get_next_run(1233097200));
}

BOOST_AUTO_TEST_CASE(IntervalOnceShort)
{
    WeekCron cron("2345",0,1,86400);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233097200), cron.get_next_run(1233097199));
}

BOOST_AUTO_TEST_CASE(IntervalTooShort)
{
    WeekCron cron("2345",0,0,86400);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233183600), cron.get_next_run(1233097200));
}

BOOST_AUTO_TEST_CASE(IntervalOverDSTBackwards1)
{
    WeekCron cron("0123456",0,14400,60);
    // calc at: Sun Oct 26 01:59:59 CEST 2008
    // expected next run: Sun Oct 26 02:00:00 CEST 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224979200), cron.get_next_run(1224979199));
}

BOOST_AUTO_TEST_CASE(IntervalOverDSTBackwards2)
{
    WeekCron cron("0123456",0,14400,60);
    // calc at: Sun Oct 26 02:29:59 CEST 2008
    // expected next run: Sun Oct 26 02:30:00 CEST 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224981000), cron.get_next_run(1224980999));
}

BOOST_AUTO_TEST_CASE(IntervalOverDSTBackwards3)
{
    WeekCron cron("0123456",0,14400,60);
    // calc at: Sun Oct 26 02:59:59 CEST 2008
    // expected next run: Sun Oct 26 02:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224982800), cron.get_next_run(1224982799));
}

BOOST_AUTO_TEST_CASE(IntervalOverDSTBackwards4)
{
    WeekCron cron("0123456",0,14400,60);
    // calc at: Sun Oct 26 02:59:59 CET 2008
    // expected next run: Sun Oct 26 03:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224986400), cron.get_next_run(1224986399));
}

BOOST_AUTO_TEST_CASE(IntervalInFirstDSTBackwards1)
{
    WeekCron cron("0123456",0,9000,60);
    // calc at: Sun Oct 26 02:28:59 CEST 2008
    // expected next run: Sun Oct 26 02:29:00 CEST 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224980940), cron.get_next_run(1224980939));
}

BOOST_AUTO_TEST_CASE(IntervalInFirstDSTBackwards2)
{
    WeekCron cron("0123456",0,9000,60);
    // calc at: Sun Oct 26 02:29:01 CEST 2008
    // expected next run: Mon Oct 27 00:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225062000), cron.get_next_run(1224980941));
}

BOOST_AUTO_TEST_CASE(IntervalInSecondDSTBackwards1)
{
    WeekCron cron("0123456",0,12600,60);
    // calc at: Sun Oct 26 02:28:59 CET 2008
    // expected next run: Sun Oct 26 02:29:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224984540), cron.get_next_run(1224984539));
}

BOOST_AUTO_TEST_CASE(IntervalInSecondDSTBackwards2)
{
    WeekCron cron("0123456",0,12600,60);
    // calc at: Sun Oct 26 02:29:01 CET 2008
    // expected next run: Sun Oct 26 02:30:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224984600), cron.get_next_run(1224984541));
}

BOOST_AUTO_TEST_CASE(IntervalOutFirstDSTBackwards1)
{
    WeekCron cron("0123456",9000,18000,60);
    // calc at: Sun Oct 26 02:29:00 CEST 2008
    // expected next run: Sun Oct 26 02:30:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224984600), cron.get_next_run(1224980940));
}

BOOST_AUTO_TEST_CASE(IntervalOutFirstDSTBackwards2)
{
    WeekCron cron("0123456",9000,18000,60);
    // calc at: Sun Oct 26 02:29:00 CET 2008
    // expected next run: Sun Oct 26 02:30:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224984600), cron.get_next_run(1224984540));
}

BOOST_AUTO_TEST_CASE(IntervalOutSecondDSTBackwards1)
{
    WeekCron cron("0123456",12600,18000,60);
    // calc at: Sun Oct 26 02:29:59 CET 2008
    // expected next run: Sun Oct 26 03:30:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224988200), cron.get_next_run(1224984599));
}

BOOST_AUTO_TEST_CASE(IntervalDSTBackwardsWholeday1)
{
    WeekCron cron("0123456",0,86400,60);
    // calc at: Sun Oct 26 22:58:59 CET 2008
    // expected next run: Sun Oct 26 22:59:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225058340), cron.get_next_run(1225058339));
}

BOOST_AUTO_TEST_CASE(IntervalDSTBackwardsWholeday2)
{
    WeekCron cron("0123456",0,86400,60);
    // calc at: Sun Oct 26 22:59:59 CET 2008
    // expected next run: Sun Oct 26 23:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225058400), cron.get_next_run(1225058399));
}

BOOST_AUTO_TEST_CASE(IntervalDSTBackwardsWholeday3)
{
    WeekCron cron("0123456",0,86400,60);
    // calc at: Sun Oct 26 23:30:00 CET 2008
    // expected next run: Sun Oct 26 23:31:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225060260), cron.get_next_run(1225060200));
}

BOOST_AUTO_TEST_CASE(IntervalOverDSTForward1)
{
    WeekCron cron("0123456",0,14400,60);
    // calc at: Sun Mar 29 01:59:59 CET 2009
    // expected next run: Sun Mar 29 03:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238288400), cron.get_next_run(1238288399));
}

BOOST_AUTO_TEST_CASE(IntervalOverDSTForward2)
{
    WeekCron cron("0123456",0,14400,60);
    // calc at: Sun Mar 29 03:58:59 CEST 2009
    // expected next run: Sun Mar 29 03:59:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238291940), cron.get_next_run(1238291939));
}

BOOST_AUTO_TEST_CASE(IntervalOverDSTForward3)
{
    WeekCron cron("0123456",0,14400,60);
    // calc at: Sun Mar 29 03:59:59 CEST 2009
    // expected next run: Mon Mar 30 00:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238364000), cron.get_next_run(1238291999));
}

BOOST_AUTO_TEST_CASE(IntervalInDSTForward1)
{
    WeekCron cron("0123456",3600,9000,60);
    // calc at: Sun Mar 29 01:59:59 CET 2009
    // expected next run: Sun Mar 29 03:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238288400), cron.get_next_run(1238288399));
}

BOOST_AUTO_TEST_CASE(IntervalInDSTForward2)
{
    WeekCron cron("0123456",3600,9000,60);
    // calc at: Sun Mar 29 03:29:59 CEST 2009
    // expected next run: Mon Mar 30 01:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238367600), cron.get_next_run(1238290199));
}

BOOST_AUTO_TEST_CASE(IntervalInDSTForward3)
{
    WeekCron cron("0123456",7200,9000,60);
    // calc at: Sun Mar 29 01:59:59 CET 2009
    // expected next run: Sun Mar 29 03:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238288400), cron.get_next_run(1238288399));
}

BOOST_AUTO_TEST_CASE(IntervalOutDSTForward)
{
    WeekCron cron("0123456",9000,14400,60);
    // calc at: Sun Mar 29 03:29:59 CEST 2009
    // expected next run: Sun Mar 29 03:30:30 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238290200), cron.get_next_run(1238290199));
}

BOOST_AUTO_TEST_CASE(IntervalDuringDSTForward)
{
    WeekCron cron("0123456",7200,10800,60);
    // calc at: Sun Mar 29 01:59:59 CET 2009
    // expected next run: Sun Mar 29 03:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238288400), cron.get_next_run(1238288399));
}

BOOST_AUTO_TEST_CASE(IntervalDSTForwardWholeday1)
{
    WeekCron cron("03",0,86400,60);
    // calc at: Sun Mar 29 23:59:59 CEST 2009
    // expected next run: Wed Apr  1 00:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238536800), cron.get_next_run(1238363999));
}

BOOST_AUTO_TEST_CASE(IntervalDSTForwardWholeday2)
{
    WeekCron cron("03",0,86400,60);
    // calc at: Mon Mar 30 00:59:59 CEST 2009
    // expected next run: Wed Apr  1 00:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238536800), cron.get_next_run(1238367599));
}

BOOST_AUTO_TEST_CASE(IntervalDSTForwardWholeday3)
{
    WeekCron cron("0123456",0,86340,60);
    // calc at: Sun Mar 29 11:42:06 CEST 2009
    // expected next run: Sun Mar 29 11:43:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238319780), cron.get_next_run(1238319726));
}

BOOST_AUTO_TEST_CASE(IntervalDSTForwardWholeday4)
{
    WeekCron cron("0123456",3599,86340,60);
    // calc at: Sun Mar 29 11:42:06 CEST 2009
    // expected next run: Sun Mar 29 11:42:59 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238319779), cron.get_next_run(1238319726));
}

BOOST_AUTO_TEST_CASE(IntervalDSTForwardWholeday5)
{
    WeekCron cron("0123456",3600,86340,60);
    // calc at: Sun Mar 29 11:42:06 CEST 2009
    // expected next run: Sun Mar 29 11:43:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238319780), cron.get_next_run(1238319726));
}

BOOST_AUTO_TEST_SUITE_END()
